<?php
/**
 * WooCommerce Print Invoices/Packing Lists
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@skyverge.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Print
 * Invoices/Packing Lists to newer versions in the future. If you wish to
 * customize WooCommerce Print Invoices/Packing Lists for your needs please refer
 * to http://docs.woothemes.com/document/woocommerce-print-invoice-packing-list/
 *
 * @package   WC-Print-Invoices-Packing-Lists/Admin
 * @author    SkyVerge
 * @copyright Copyright (c) 2011-2016, SkyVerge, Inc.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

defined( 'ABSPATH' ) or exit;

/**
 * factor Customizer class
 *
 * Handles WYSIWYG customization of the document templates
 *
 * @since 3.0.0
 */
class WC_factor_Customizer {


	/** @var string The customizer trigger */
	private $customizer_trigger = '';

	/** @var string The customizer URL address */
	private $customizer_url = '';


	/**
	 * factor Document Customizer Constructor
	 *
	 * @since 3.0.0
	 */
	public function __construct() {

		$this->customizer_trigger = $this->get_customizer_trigger();
		$this->customizer_url     = $this->get_customizer_url();

		// register a new setting field type
		add_action( 'woocommerce_admin_field_wc_factor_customizer_button', array( $this, 'render_customize_button_html' ) );

		// add a Customize button to access the customizer from factor Settings
		add_filter( 'wc_factor_general_settings', array( $this, 'add_customize_button' ) );

		// register factor customizer settings
		add_filter( 'customize_register', array( $this, 'add_customizer_settings' ) );

		// load factor customizer content when invoked
		if ( isset( $_GET[ $this->customizer_trigger ] ) ) {

			// first remove stuff we don't need on our views...
			remove_action( 'wp_footer', 'wp_admin_bar_render', 1000 );
			add_action( 'wp_enqueue_scripts', array( $this, 'dequeue_theme_styles' ), 1000 );
			add_filter( 'customize_register', array( $this, 'remove_sections' ), 150 );
			// ...then add our stuff
			add_filter( 'customize_register', array( $this, 'add_customizer_sections' ), 200 );
			add_filter( 'customize_register', array( $this, 'add_customizer_controls' ), 200 );

			// enqueue Customizer scripts and styles
			add_action( 'customize_controls_enqueue_scripts', array( $this, 'customizer_controls_scripts' ), 999 );
			add_action( 'customize_preview_init',             array( $this, 'customizer_preview_scripts' ), 999 );
			add_action( 'customize_controls_print_styles',    array( $this, 'customizer_styles' ), 999 );
		}

		// Load the factor Customizer template
		add_action( 'template_redirect', array( $this, 'load_template' ), 1 );
	}


	/**
	 * Get the customizer action trigger
	 *
	 * @since 3.0.0
	 * @return string
	 */
	public function get_customizer_trigger() {
		return 'wc-factor-customizer';
	}


	/**
	 * Get the customizer url
	 *
	 * @since 3.0.0
	 * @return string
	 */
	public function get_customizer_url() {

		// Build URL
		$url = admin_url( 'customize.php' );
		$url = add_query_arg( $this->customizer_trigger, 'true', $url );
		$url = add_query_arg( 'url', wp_nonce_url( site_url() . '/?' . $this->customizer_trigger .'=true', 'preview-document' ), $url );
		$url = add_query_arg( 'return', urlencode( add_query_arg( array( 'page' => 'wc-settings', 'tab' => 'factor' ), admin_url( 'admin.php' ) ) ), $url );

		return esc_url_raw( $url );
	}


	/**
	 * Add a custom setting to display a customizer button
	 *
	 * @since 3.0.0
	 * @param array $settings
	 */
	public function render_customize_button_html( $settings ) {
		?>
		<tr valign="top">
			<th scope="row" class="titledesc"><?php echo esc_html( $settings['desc'] ); ?></th>
			<td class="forminp forminp-<?php echo sanitize_html_class( $settings['type'] ) ?>">
				<a href="<?php echo $settings['link']; ?>">
					<button
						name="<?php echo esc_attr( $settings['id'] ); ?>"
						id="<?php echo esc_attr( $settings['id'] ); ?>"
						style="<?php echo esc_attr( $settings['css'] ); ?>"
						class="button-primary <?php echo esc_attr( $settings['class'] ); ?>"
						type="button">
						<?php echo $settings['title']; ?>
					</button>
				</a>
			</td>
		</tr>
		<?php
	}


	/**
	 * Add Customize Button to factor Settings
	 *
	 * @since 3.0.0
	 * @param array $template_settings Template Settings
	 * @return array
	 */
	public function add_customize_button( $template_settings ) {

		$customizer_settings = array(
			// start section
			array(
				'title' => __( 'Template Customizer', 'woocommerce-factor' ),
				'desc'  => __( 'Customize the template using a visual editor.', 'woocommerce-factor' ),
				'type'  => 'title',
			),
			// add the button
			array(
				'title' => __( 'Customize', 'woocommerce-factor'),
				'desc'  => __( 'Customize Template', 'woocommerce-factor' ),
				'type'  => 'wc_factor_customizer_button',
				'id'    => 'wc_factor_template_customizer_button',
				'link'  => $this->customizer_url,
			),
			// end section
			array(
				'type' => 'sectionend',
				'id'   => 'email_customizer_sectionend',
			),
		);

		return array_merge( $customizer_settings, $template_settings );
	}


	/**
	 * Remove Customizer Sections
	 *
	 * @since 3.0.0
	 * @param WP_Customize_Manager $wp_customize Customizer instance
	 */
	public function remove_sections( $wp_customize ) {
		global $wp_customize;

		// remove all sections
		if ( $wp_customize instanceof WP_Customize_Manager && $sections = $wp_customize->sections() ) {

			foreach ( $sections as $section_id => $section ) {
				$wp_customize->remove_section( $section_id );
			}
		}
	}


	/**
	 * Add Customizer Sections
	 *
	 * @since 3.0.0
	 * @param WP_Customize_Manager $wp_customize Customizer instance
	 */
	public function add_customizer_sections( $wp_customize ) {
		global $wp_customize;

		// company
		$wp_customize->add_section( 'wc_factor_company_information', array (
			'title'      => __( 'Company Information', 'woocommerce-factor' ),
			'capability' => 'manage_woocommerce',
			'priority'   => 10,
		) );

		// typography
		$wp_customize->add_section( 'wc_factor_typography', array (
			'title'      => __( 'Typography', 'woocommerce-factor' ),
			'capability' => 'manage_woocommerce',
			'priority'   => 15,
		) );

		// colors
		$wp_customize->add_section( 'wc_factor_colors', array (
			'title'      => __( 'Colors', 'woocommerce-factor' ),
			'capability' => 'manage_woocommerce',
			'priority'   => 20,
		) );

		// content
		$wp_customize->add_section( 'wc_factor_content', array (
			'title'      => __( 'Content', 'woocommerce-factor' ),
			'capability' => 'manage_woocommerce',
			'priority'   => 25,
		) );

		// misc & advanced
		$wp_customize->add_section( 'wc_factor_misc', array (
			'title'      => __( 'Advanced', 'woocommerce-factor' ),
			'capability' => 'manage_woocommerce',
			'priority'   => 30,
		) );

	}


	/**
	 * Add Customizer Settings
	 *
	 * @since 3.0.0
	 * @param WP_Customize_Manager $wp_customize Customizer instance
	 */
	public function add_customizer_settings( $wp_customize ) {
		global $wp_customize;


		// ===================
		// Company Information
		// ===================

		// company logo
		$wp_customize->add_setting( 'wc_factor_company_logo', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// logo max size
		$wp_customize->add_setting( 'wc_factor_company_logo_max_width', array(
			'type'      => 'option',
			'default'   => '300',
			'transport' => 'postMessage',
		) );

		// company name
		$wp_customize->add_setting( 'wc_factor_company_name', array(
			'type'      => 'option',
			'default'   => get_bloginfo( 'name' ),
			'transport' => 'postMessage',
		) );

		// company URL
		$wp_customize->add_setting( 'wc_factor_company_url', array(
			'type'      => 'option',
			'default'   => get_bloginfo( 'url' ),
			'transport' => 'postMessage',
		) );

		// company extra
		$wp_customize->add_setting( 'wc_factor_company_extra', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// company title alignment
		$wp_customize->add_setting( 'wc_factor_company_title_align', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// company address
		$wp_customize->add_setting( 'wc_factor_company_address', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// company address alignment
		$wp_customize->add_setting( 'wc_factor_company_address_align', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );


		// ==========
		// Typography
		// ==========

		// body font size
		$wp_customize->add_setting( 'wc_factor_body_font_size', array(
			'type'      => 'option',
			'default'   => '14',
			'transport' => 'postMessage',
		) );

		// heading font size
		$wp_customize->add_setting( 'wc_factor_heading_font_size', array(
			'type'      => 'option',
			'default'   => '32',
			'transport' => 'postMessage',
		) );


		// ======
		// Colors
		// ======

		// link color
		$wp_customize->add_setting( 'wc_factor_link_color', array(
			'type'      => 'option',
			'default'   => '#000000',
			'transport' => 'postMessage',
		) );

		// headings color
		$wp_customize->add_setting( 'wc_factor_headings_color', array(
			'type'      => 'option',
			'default'   => '#000000',
			'transport' => 'postMessage',
		) );

		// table head background color
		$wp_customize->add_setting( 'wc_factor_table_head_bg_color', array(
			'type'      => 'option',
			'default'   => '#333333',
			'transport' => 'postMessage',
		) );

		// table head background color
		$wp_customize->add_setting( 'wc_factor_table_head_color', array(
			'type'      => 'option',
			'default'   => '#FFFFFF',
			'transport' => 'postMessage',
		) );


		// =======
		// Content
		// =======

		// header
		$wp_customize->add_setting( 'wc_factor_header', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// terms & conditions
		$wp_customize->add_setting( 'wc_factor_return_policy', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// terms & conditions fine print
		$wp_customize->add_setting( 'wc_factor_return_policy_fine_print', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

		// footer
		$wp_customize->add_setting( 'wc_factor_footer', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );


		// ========
		// ADVANCED
		// ========

		// custom CSS
		$wp_customize->add_setting( 'wc_factor_custom_styles', array(
			'type'      => 'option',
			'transport' => 'postMessage',
		) );

	}


	/**
	 * Add Customizer Controls
	 *
	 * @since 3.0.0
	 * @param WP_Customize_Manager $wp_customize Customizer instance
	 */
	public function add_customizer_controls( $wp_customize ) {
		global $wp_customize;


		// ===================
		// Company Information
		// ===================

		// company logo
		$wp_customize->add_control( new WP_Customize_Image_Control( $wp_customize, 'wc_factor_shop_logo', array(
			'label'       => __( 'Shop Logo', 'woocommerce-factor' ),
			'description' => __( 'Upload a logo representing your shop or business.', 'woocommerce-factor' ),
			'priority'    => 10,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_logo',
			'context'     => 'woocommerce_factor_customizer',
		) ) );

		// logo max width
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_logo_max_width_control', array(
			'type'        => 'range',
			'priority'    => 15,
			'section'     => 'wc_factor_company_information',
			'label'       => __( 'Logo Max Width', 'woocommerce-factor' ),
			'description' => __( 'The maximum width of the logo.', 'woocommerce-factor' ),
			'settings'    => 'wc_factor_company_logo_max_width',
			'input_attrs' => array(
				'min'   => 0,
				'max'   => 800,
				'step'  => 10,
			),
		) ) );

		// company name
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_name_control', array(
			'label'       => __( 'Shop Name', 'woocommerce-factor' ),
			'description' => __( 'Enter your shop or business name to be displayed when not using a logo. Leave blank to use the default site title.', 'woocommerce-factor' ),
			'priority'    => 20,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_name',
			'type'        => 'text',
		) ) );

		// company URL
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_url_control', array(
			'label'       => __( 'Shop URL', 'woocommerce-factor' ),
			'description' => __( 'Enter the URL to be used in the template for your shop or business website URL. Leave blank to use the default site url.', 'woocommerce-factor' ),
			'priority'    => 25,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_url',
			'type'        => 'text',
		) ) );

		// extra information
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_extra_control', array(
			'label'       => __( 'Extra information', 'woocommerce-factor' ),
			'description' => __( 'Enter additional information or subheading to be displayed below the title or logo.', 'woocommerce-factor' ),
			'priority'    => 30,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_extra',
			'type'        => 'textarea',
		) ) );

		// company title alignment
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_title_align_control', array(
			'label'       => __( 'Shop name or logo alignment', 'woocommerce-factor' ),
			'description' => __( 'Set the alignment of the shop title or logo in template.', 'woocommerce-factor' ),
			'priority'    => 35,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_title_align',
			'type'        => 'select',
			'default'     => 'left',
			'choices'     => array(
				'left'   => __( 'Left', 'woocommerce-factor' ),
				'right'  => __( 'Right', 'woocommerce-factor' ),
				'center' => __( 'Center', 'woocommerce-factor' ),
			),
		) ) );

		// company address
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_address_control', array(
			'label'       => __( 'Shop Address', 'woocommerce-factor' ),
			'description' => __( 'Enter your shop or business address to be displayed next to the logo or shop name.', 'woocommerce-factor' ),
			'priority'    => 40,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_address',
			'type'        => 'textarea',
		) ) );

		// company address align
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_shop_address_align_control', array(
			'label'       => __( 'Shop Address alignment', 'woocommerce-factor' ),
			'description' => __( 'Set the alignment of the shop address in template.', 'woocommerce-factor' ),
			'priority'    => 45,
			'section'     => 'wc_factor_company_information',
			'settings'    => 'wc_factor_company_address_align',
			'type'        => 'select',
			'default'     => 'right',
			'choices'     => array(
				'left'   => __( 'Left', 'woocommerce-factor' ),
				'right'  => __( 'Right', 'woocommerce-factor' ),
				'center' => __( 'Center', 'woocommerce-factor' ),
			),
		) ) );


		// ==========
		// Typography
		// ==========

		// body font
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_body_font_size_control', array(
			'type'        => 'range',
			'priority'    => 15,
			'section'     => 'wc_factor_typography',
			'label'       => __( 'Body font size', 'woocommerce-factor' ),
			'description' => __( 'The template default body font size.', 'woocommerce-factor' ),
			'settings'    => 'wc_factor_body_font_size',
			'default'     => 12,
			'input_attrs' => array(
				'min'   => 1,
				'max'   => 36,
				'step'  => 1,
			),
		) ) );

		// heading font
		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_heading_font_size_control', array(
			'type'        => 'range',
			'priority'    => 25,
			'section'     => 'wc_factor_typography',
			'label'       => __( 'Heading font size', 'woocommerce-factor' ),
			'description' => __( 'The template default heading font size.', 'woocommerce-factor' ),
			'settings'    => 'wc_factor_heading_font_size',
			'default'     => 28,
			'input_attrs' => array(
				'min'   => 1,
				'max'   => 96,
				'step'  => 1,
			),
		) ) );


		// ======
		// Colors
		// ======

		// link color
		$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, 'wc_factor_link_color_control', array(
			'label'       => __( 'Link Color', 'woocommerce-factor' ),
			'description' => __( 'Choose the default color of links in template.', 'woocommerce-factor' ),
			'priority'    => 15,
			'section'     => 'wc_factor_colors',
			'settings'    => 'wc_factor_link_color',
		) ) );

		// heading color
		$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, 'wc_factor_headings_color_control', array(
			'label'       => __( 'Headings Color', 'woocommerce-factor' ),
			'description' => __( 'Choose the default color for heading text.', 'woocommerce-factor' ),
			'priority'    => 20,
			'section'     => 'wc_factor_colors',
			'settings'    => 'wc_factor_headings_color',
		) ) );

		// table head background color
		$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, 'wc_factor_table_head_bg_color_control', array(
			'label'       => __( 'Table Head Background Color', 'woocommerce-factor' ),
			'description' => __( 'Choose the default color for the table head background.', 'woocommerce-factor' ),
			'priority'    => 25,
			'section'     => 'wc_factor_colors',
			'settings'    => 'wc_factor_table_head_bg_color',
		) ) );

		// table head color
		$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, 'wc_factor_table_head_color_control', array(
			'label'       => __( 'Table Head Color', 'woocommerce-factor' ),
			'description' => __( 'Choose the default color for the table head text.', 'woocommerce-factor' ),
			'priority'    => 25,
			'section'     => 'wc_factor_colors',
			'settings'    => 'wc_factor_table_head_color',
		) ) );


		// =======
		// Content
		// =======

		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_header_control', array(
			'label'       => __( 'Header', 'woocommerce-factor' ),
			'description' => __( "Enter additional information to be displayed below the document's header.", 'woocommerce-factor' ),
			'priority'    => 10,
			'section'     => 'wc_factor_content',
			'settings'    => 'wc_factor_header',
			'type'        => 'textarea',
		) ) );

		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'woocommerce_factor_return_policy_control', array(
			'label'       => __( 'Returns Policy, Terms and Conditions, etc.', 'woocommerce-factor' ),
			'description' => __( "Enter your shop's policies to be displayed at the bottom of the document.", 'woocommerce-factor' ),
			'priority'    => 20,
			'section'     => 'wc_factor_content',
			'settings'    => 'wc_factor_return_policy',
			'type'        => 'textarea',
		) ) );

		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'woocommerce_factor_return_policy_fine_print_control', array(
			'label'       => __( 'Fine print', 'woocommerce-factor' ),
			'description' => __( 'Display the terms and conditions text in fine print.', 'woocommerce-factor' ),
			'priority'    => 25,
			'section'     => 'wc_factor_content',
			'settings'    => 'wc_factor_return_policy_fine_print',
			'type'        => 'checkbox',
		) ) );

		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_footer_control', array(
			'label'       => __( 'Footer', 'woocommerce-factor' ),
			'description' => __( "Enter additional information to be displayed at the document's footer.", 'woocommerce-factor' ),
			'priority'    => 30,
			'section'     => 'wc_factor_content',
			'settings'    => 'wc_factor_footer',
			'type'        => 'textarea',
		) ) );


		// ========
		// Advanced
		// ========

		$wp_customize->add_control( new WP_Customize_Control( $wp_customize, 'wc_factor_custom_styles_control', array(
			'label'       => __( 'Custom Styles', 'woocommerce-factor' ),
			'description' => __( 'For advanced users: you may enter additional CSS styles that will be used in the document template.', 'woocommerce-factor' ),
			'priority'    => 20,
			'section'     => 'wc_factor_misc',
			'settings'    => 'wc_factor_custom_styles',
			'type'        => 'textarea',
		) ) );
	}


	/**
	 * Add customizer scripts for customizing controls
	 *
	 * @since 3.0.0
	 */
	public function customizer_controls_scripts() {

		// Prevent script caching in development environments
		$version = defined( 'WP_DEBUG' ) && WP_DEBUG ? WC_factor::VERSION . '.' . time() : WC_factor::VERSION;

		wp_enqueue_script( 'wc-factor-customizer-controls-scripts',  wc_factor()->get_plugin_url() . '/assets/js/admin/wc-factor-customizer-controls.min.js', array( 'jquery' ), $version, true );

		wp_localize_script( 'wc-factor-customizer-controls-scripts', 'wc_factor_customizer', array(
			'i18n' => array(
				'preview_changes' => __( 'Save and Preview Changes', 'woocommerce-factor' ),
			),
		) );
	}


	/**
	 * Add customizer scripts for previewing changes
	 *
	 * @since 3.0.0
	 */
	public function customizer_preview_scripts() {

		// Prevent script caching in development environments
		$version = defined( 'WP_DEBUG' ) && WP_DEBUG ? WC_factor::VERSION . '.' . time() : WC_factor::VERSION;

		wp_enqueue_script( 'wc-factor-customizer-preview-scripts',  wc_factor()->get_plugin_url() . '/assets/js/admin/wc-factor-customizer-preview.min.js', array( 'jquery', 'customize-preview' ), $version, true );
	}


	/**
	 * Add Customizer styles
	 *
	 * @since 3.0.0
	 */
	public function customizer_styles() {
		?>
		<style type="text/css">
			#accordion-panel-widgets,
			#accordion-panel-nav_menus {
				display: none !important;
				height: 0 !important;
				visibility: hidden !important;
			}
			span.wc-factor-range-index {
				color: #999999;
				display: block;
				font-style: italic;
				margin-top: 4px;
			}
		</style>
		<?php
	}

	/**
	 * Dequeue theme styles from Customizer preview
	 *
	 * @since 3.0.0
	 */
	public function dequeue_theme_styles() {
		global $wp_styles;

		if ( ! is_object( $wp_styles ) || ! isset( $_GET[ $this->customizer_trigger ] ) ) {
			return;
		}

		foreach ( $wp_styles->registered as $registered ) {

			if ( is_object( $registered ) && ( SV_WC_Helper::str_ends_with( get_template_directory_uri() . '/style.css', $registered->src ) || SV_WC_Helper::str_ends_with( get_stylesheet_uri(), $registered->src ) ) ) {
				wp_deregister_style( $registered->handle );
			}
		}
	}


	/**
	 * Load template preview
	 *
	 * @since 3.0.0
	 * @param WP_Query $wp_query
	 * @return void|WP_Query Maybe redirect
	 */
	public function load_template( $wp_query ) {

		if ( isset( $_GET[ $this->customizer_trigger ] ) ) {

			wp_head();

			ob_start();

			include( wc_factor()->get_plugin_path() . '/includes/admin/views/html-factor-template-preview.php' );

			$template = ob_get_clean();

			wp_footer();

			echo $template;
			exit;
		}

		return $wp_query;
	}


}
